%----------------------------------------------------------------------------
% Run this script in Matlab command window
%
% Compile C-language Mex files
% sfmult  -- a sparse A times a dense matrix
% dfeast  -- mex file to call feast
%
% This file is based on cholmod_make in suitesparse
%   Timothy A. Davis, http://www.suitesparse.com
%
%
% Zaiwen Wen
% 07-28-2014
%


function Installmex

src = pwd;
sdet = 'src';
fname{1} = 'sfmult'; fcc{1} = 'cpp';
fname{2} = 'dfeast'; fcc{2} = 'cpp';

hasMKL = 1; % with MKL or not
%-------------------------------------
% modify the paths to MKL if needed
MKLHOMEINCLUDE = '/opt/intel/composer_xe_2015.3.187/mkl/include';
MKLHOMELIB = '/opt/intel/composer_xe_2015.3.187/mkl/lib';

% MKLHOMEINCLUDE = '/opt/intel/mkl/include';
% MKLHOMELIB = '/opt/intel/mkl/lib/intel64';
%-------------------------------------


details = 0 ;	    % 1 if details of each command are to be printed

v = version ;
try
    % ispc does not appear in MATLAB 5.3
    pc = ispc ;
    mac = ismac ;
catch                                                                       %#ok
    % if ispc fails, assume we are on a Windows PC if it's not unix
    pc = ~isunix ;
    mac = 0 ;
end

% if (~pc) && (~mac)
%     mex -O -largeArrayDims -lmwlapack -lmwblas  sfmult.cpp
%     mex -O -largeArrayDims -lmwlapack -lmwblas  dfeast.cpp
%     return
% end

flags = '' ;
is64 = ~isempty (strfind (computer, '64')) ;
if (is64)
    % 64-bit MATLAB
    flags = '-largeArrayDims' ;
end

% MATLAB 8.3.0 now has a -silent option to keep 'mex' from burbling too much
if (~verLessThan ('matlab', '8.3.0'))
    flags = ['-silent ' flags] ;
end

 %---------------------------------------------------------------------------
 % BLAS option
 %---------------------------------------------------------------------------

 % This is exceedingly ugly.  The MATLAB mex command needs to be told where to
 % fine the LAPACK and BLAS libraries, which is a real portability nightmare.

if (pc)
    if (verLessThan ('matlab', '6.5'))
        % MATLAB 6.1 and earlier: use the version supplied here
        lapack = 'lcc_lib/libmwlapack.lib' ;
    elseif (verLessThan ('matlab', '7.5'))
        lapack = 'libmwlapack.lib' ;
    else
        lapack = 'libmwlapack.lib libmwblas.lib' ;
    end
else
    if (verLessThan ('matlab', '7.5'))
        lapack = '-lmwlapack' ;
    else
        lapack = '-lmwlapack -lmwblas' ;
    end
end

if (is64 && ~verLessThan ('matlab', '7.8'))
    % versions 7.8 and later on 64-bit platforms use a 64-bit BLAS
    fprintf ('with 64-bit BLAS\n') ;
    flags = [flags ' -DBLAS64'] ;
end

if (~(pc || mac))
    % for POSIX timing routine
    lapack = [lapack ' -lrt'] ;
end

include = '';
mkl = '';
if hasMKL
    include = ['-I',MKLHOMEINCLUDE];
    if mac
        mkl = [' ', MKLHOMELIB,filesep,'libmkl_intel_lp64.dylib '];
        mkl = [mkl, ' ', MKLHOMELIB,filesep,'libmkl_core.dylib '];
        mkl = [mkl, ' ', MKLHOMELIB,filesep,'libmkl_intel_thread.dylib   -ldl  -lm '];
%         mkl = [' ', MKLHOMELIB,filesep,'libmkl_intel_lp64.a '];
%         mkl = [mkl, ' ', MKLHOMELIB,filesep,'libmkl_core.a '];
%         mkl = [mkl, ' ', MKLHOMELIB,filesep,'libmkl_intel_thread.a   -ldl  -lm  '];

        %     else
%         mkl = [' ', MKLHOMELIB,filesep,'libmkl_intel_lp64.so '];
%         mkl = [mkl, ' ', MKLHOMELIB,filesep,'libmkl_core.so '];
%         mkl = [mkl, ' ', MKLHOMELIB,filesep,'libmkl_intel_thread.so   -ldl  -lm  -lrt'];
    end
end
    
if (verLessThan ('matlab', '7.0'))
    % do not attempt to compile CHOLMOD with large file support
    include = [include ' -DNLARGEFILE'] ;
elseif (~pc)
    % Linux/Unix require these flags for large file support
    include = [include ' -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE'] ;
end

if (verLessThan ('matlab', '6.5'))
    % logical class does not exist in MATLAB 6.1 or earlie
    include = [include ' -DMATLAB6p1_OR_EARLIER'] ;
end

cd(sdet);
% compile each mexFunction
for k = 1:length(fname)
    s = sprintf ('mex %s -DDLONG -O %s %s.%s', flags, ...
            include, fname{k}, fcc{k}) ;
    s = [s ' ' lapack mkl] ;						    %#ok
    %s = [s ' '  mkl] ;
    cmd (s, details) ;
end
cd(src);

 %------------------------------------------------------------------------------
function  cmd (s, details)
 %DO_CMD: evaluate a command, and either print it or print a "."
if (details)
    fprintf ('%s\n', s) ;
end
eval (s) ;


