function test_prob_in_eigpen_MPM(maxeig, SpMV_MKL, solvers, opts, prefix, tol)
% 
% a test script for comparison
% summary of the results are saved in txt files at the folder "results"
%
% Zaiwen Wen
% July   28, 2015
%-------------------------------------------------------
% path of test matrices
addpath('matrix'); % local path

% path at Edison
addpath('/global/u2/z/zaiwen/matrix/matbin');
addpath('/global/u2/z/zaiwen/matrix/PARSEC');

% path at workstation
addpath('/opt/code/matrix/matbin');
addpath('/opt/code/matrix/PARSEC');

% path at Mac OS
addpath('/Users/wenzw/code/matrix/matbin');
addpath('/Users/wenzw/code/matrix/PARSEC');
% -------------------------------------------------------


% compute largest or smallest eigenvalues
if nargin <= 0;  maxeig = 1;       end
if nargin <= 1;  SpMV_MKL = 0;     end
if nargin <= 2;  solvers = [0, 0, 0, 0, 1, 0, 1]; end
if nargin <= 3;  opts = [];        end

%add prefix to filename so that the previous results will not be over-written
if nargin <= 4;  prefix = '';      end
if nargin <= 5;  tol = 1e-12;      end

% select a solver
run_EIGS  = solvers(1);
run_feast = solvers(2);
run_filtlan = solvers(3);
run_laneig = solvers(4);
run_mpm = solvers(5);
run_GN = solvers(6);
run_primme = solvers(7);

% select tolerance
% tol = 1e-6;
% tol = 1e-8;
% tol = 1e-12;

% print or not.
printlev = 0;  % 0 - all information, 1 - summary
stopRule = 2;

% list of matrices
prob = { 'Andrews', 'C60',  'cfd1', ...  % 1-3
    'finance', 'Ga10As10H30', 'Ga3As3H12', 'OPF3754', ... %4-7
    'shallow_water1s', 'Si10H16', 'Si5H12', 'SiO', 'wathen100', ... % 8 -12
    'Ge87H76', 'Ge99H100', 'Si41Ge41H72', 'Si87H76', 'Ga41As41H72'}; % 13 -17

klist =[600, 176, 707, 748, 1000, 613, 154, 819, 171, 199, 334, 304, 1000, 1000, 1000, 1000, 1000];

maxeig_int = [2.471456e+01, 3.658528e+01 
8.659796e+00, 9.628692e+00 
4.235893e+00, 6.902753e+00 
1.989466e+01, 2.828332e+01 
5.299920e+01, 1.300904e+03 
9.335378e+01, 1.299981e+03 
3.060182e+04, 1.085079e+05 
3.292640e+00, 3.613871e+00 
3.318897e+01, 3.702295e+01 
5.248709e+01, 5.866094e+01 
7.562148e+01, 8.441391e+01 
2.991369e+02, 3.697741e+02 
2.953392e+01, 3.286409e+01 
2.949168e+01, 3.280306e+01 
4.614959e+01, 4.991855e+01 
4.036237e+01, 4.317456e+01 
5.540556e+01, 1.301034e+03];

mineig_int = [-2.694199e+00, 1.000000e-01 
-5.853224e-02, 1.144037e+01 
-1.078272e-02, 9.997995e-02 
-1.388865e+00, -8.474684e-01 
-1.687720e+00, 1.313159e+00 
-3.457997e+00, 1.262826e+00 
3.384333e+04, 1.071238e+05 
-1.039444e+00, -9.022081e-01 
-9.666410e-01, 1.248122e+00 
-1.970850e+00, 1.096202e+00 
-3.018759e+00, 1.774492e+00 
-5.060229e+00, 3.643858e-02 
-6.814085e-01, 1.314019e+00 
-6.101708e-01, 1.326417e+00 
-7.381422e-01, 1.313583e+00 
-5.485681e-01, 1.296383e+00 
-7.146397e-01, 1.350202e+00];

% select matrices
nlen = length(prob);
problist = [1:nlen];
% problist = [1:4];
% problist = [13];
% % problist = [5:nlen];
problist = [2];
%problist = [7];
%problist = [8];
%problist = [11:12];
% problist = [9:12];
% problist = [6];
% problist = [5:nlen];
% problist = [2,9:12];
% problist = [17];
%--------------------------------------------------------------------------
nprob = length(problist);

% counting the number of mat-vec products
global nAX Aop Bop

% fix random seeds
seed = 2012;
if exist('RandStream','file')
    RandStream.setGlobalStream(RandStream('mt19937ar','seed',seed));
else
    rand('state',seed); randn('state',seed);
end

% mkdir
[HOME,name,ext] = fileparts(strcat(mfilename('fullpath'),'.m'));
datadir = 'results';
if ~exist(strcat(HOME,filesep,datadir),'dir')
    mkdir(strcat(HOME,filesep,datadir));
end

eigstr = 'maxeig'; if maxeig == -1; eigstr = 'mineig'; end

SpMVstr = 'SpMV_Matlab'; if SpMV_MKL == 1; SpMVstr = 'SpMV_MKL'; end
fid = fopen(strcat('results',filesep,'prob-MPM-res-',eigstr, ...
    '-',SpMVstr, prefix, '.txt'), 'a+');

fids = fopen(strcat('results',filesep,'prob-MPM-stat-',eigstr, ...
    '-',SpMVstr, prefix, '.txt'), 'a+');

fids2 = fopen(strcat('results',filesep,'prob-MPM-stat2-',eigstr, ...
    '-',SpMVstr, prefix, '.txt'), 'a+');

fprintf(fid, '\n\n'); fprintf(fid, 'tol: %e: \n', tol);
fprintf(fids, '\n\n'); fprintf(fids, 'tol: %e: \n', tol);
fprintf(fids2, '\n\n'); fprintf(fids2, 'tol: %e: \n', tol);

for dprob = 1:nprob
    pid = problist(dprob);
    name = prob{pid};
    %k = nevlist(pid);

    namess = strrep(name,'_','\_'); 

    switch name
        case {'C60'}
            load(name);
        otherwise
            load(name, 'Problem');
            B = Problem.A;
    end
    
    clear Problem;
    n = size(B,2);
    %fprintf('%20s, n: %10d\n', name, n); continue;
    %k = nevlist(pid);
    k = max(min(round(0.01*n),1000),1);
    
    %B = sparse(B);
    if maxeig==-1; B = -B; end
    if ~SpMV_MKL; Aop = @(X) B*X; else Aop = @(X) sfmult(B,X,1); end

    sparsity = nnz(B)/n^2;
    fprintf('\nname: %s, n: %d, k: %d, sparsity: %e\n', name, n, k, sparsity);
    %continue;
    fprintf(fid,'\nname: %s, n: %d, k: %d, sparsity: %e\n', name, n, k, sparsity);
    fprintf(fids,'%15s & %5d &', namess, k);
    fprintf(fids2,'%15s & %5d ', namess, k);

    X0 = rand(n,k);
    
    if run_EIGS
        opteigs.tol = tol; opteigs.isreal = 1; opteigs.issym = 1;
        nAX = 0;
        t0 = tic;
        %[U1,D1] = eigs(A,k,etype,opteigs);
        [U1,D1] = eigs(@funA,n,k,'LA',opteigs);
        tt1 = toc(t0); t1(dprob) = tt1;
        
        [eveigs,is] = sort(diag(D1),'descend');
        
        r1 = comp_stat_eig(B, U1, stopRule, 1); % largest eigenvalues of B
        
        fprintf('EIGS res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
             r1.maxres, r1.tr, tt1, nAX);
         
        fprintf(fid,'EIGS res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
             r1.maxres, r1.tr, tt1, nAX);         
        
        fprintf(fids,'& %2.1e & %9.8e & %8.0f ', r1.maxres, r1.tr, tt1 );         
        fprintf(fids2,' && %2.1e  &  %8.0f ', r1.maxres, tt1 );         

        fprintf('\n');
    end
    
    
    %---------------------------------------------------------------------------------------
    %fprintf('\n\n--- feast running --- \n')
    if run_feast
        
        optfeast = [];
        optfeast.quiet = 2;
        %optfeast.dErrTrace = 12;
        optfeast.dErrTrace = round(abs(log10(tol)));
        optfeast.maxStep = 10;
        optfeast.eigType = 2; % largest eigenvalues
        optfeast.kFactor = 1.2;
        m0 = optfeast.kFactor*k;
        
        if run_EIGS
            emin = eveigs(end); emax = eveigs(1)+0.1;
        else
            if maxeig == 1;
                emin = maxeig_int(pid,1);
                emax = maxeig_int(pid,2);
            else
                emin = mineig_int(pid,1);
                emax = mineig_int(pid,2);
            end
        end
        
        %[emin, emax, ln] = est_key_eigvals(B, n, k);
        fprintf('interval: [%e, %e], m0: %d\n', ...
              emin, emax, m0);  
        t0 = tic; 
        [U3, D3, res, mk, info, epsout, nAX] = dfeast(B, k, emin, emax, optfeast);
        [ev3, ii] = sort(D3, 'descend'); U3 = U3(:,ii(1:k)); ev3 = ev3(1:k);
        tt3 = toc(t0); t3(dprob) = tt3;
        
        r3 = comp_stat_eig(B, U3, stopRule, 1);
        
        fprintf('FEAST res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r3.maxres, r3.tr, tt3, nAX);
        
        fprintf(fid,'FEAST res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r3.maxres, r3.tr, tt3, nAX);
       
        fprintf(fids,'& %2.1e & %9.8e & %8.0f ', r3.maxres, r3.tr, tt3 );         
        fprintf(fids2,'&& %2.1e & %8.0f ', r3.maxres, tt3 );         

        fprintf('\n');
    end
    
    
    %---------------------------------------------------------------------------------------
    %fprintf('\n\n--- feast running --- \n')
    if run_filtlan
        
        optfiltlan = [];
        
        optfiltlan.tol = tol;
        optfiltlan.nev = k;
        
        polydeg = 10;
        basedeg = 4;
        
        nAX = 0;
        %emin = eveigs(end); emax = eveigs(1)+1;
        [emin, emax, ln] = est_key_eigvals(B, n, k);

        fprintf('interval: [%e, %e]\n', emin, emax);
        
        t0 = tic; 
        %[emin, emax, ln] = est_key_eigvals(B, n, k);
        
        [U4, D4, info4] = filtlan(B, [emin, emax], polydeg, basedeg, optfiltlan);
        %[U4, D4, info4] = filtlan(B, [emin, emax], [], [], optfiltlan);
        
        D4 = diag(D4);
        [ev4, ii] = sort(D4, 'descend'); U4 = U4(:,ii(1:k)); ev4 = ev4(1:k);
        tt4 = toc(t0); t4(dprob) = tt4;
        
        r4 = comp_stat_eig(B, U4, stopRule, 1);
        
        fprintf('FiltL res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r4.maxres, r4.tr, tt4, nAX);
        
        fprintf(fid,'FiltL res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r4.maxres, r4.tr, tt4, nAX);
       

        fprintf(fids,'& %2.1e & %9.8e & %8.0f  ', r4.maxres, r4.tr, tt4 );         
        fprintf(fids2,' && %2.1e & %8.0f ', r4.maxres, tt4 );         

        fprintf('\n');
    end    
    
    %---------------------------------------------------------------------------------------
    %fprintf('\n\n--- feast running --- \n')
    if run_laneig
        
        optlaneig = [];
        optlaneig.tol = tol;
        
        nAX = 0;
        
        t0 = tic; 
        % laneig in Filtlan
        %[U5, D5, info5] = laneig(B, k, 'LA', optlaneig);
        
        % laneig in propack
        %if maxeig== 1
        %    [U5, D5, info5] = laneig(B, k, 'AL', optlaneig);
        %elseif maxeig==-1
        %    [U5, D5, info5] = laneig(-B, k, 'AS', optlaneig);     
        %end
        %[U5, D5, info5] = laneig(@funA, n, k, 'AL', optlaneig);
        
        [U5, D5, info5] = laneig(B, k, 'AL', optlaneig);
        
        tt5 = toc(t0); t5(dprob) = tt5;
        
        r5 = comp_stat_eig(B, U5, stopRule, 1);
        
        fprintf('LANEIG res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r5.maxres, r5.tr, tt5, nAX);
        
        fprintf(fid,'LANEIG res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r5.maxres, r5.tr, tt5, nAX);
       
        fprintf(fids,'& %2.1e & %9.8e & %8.0f  ', r5.maxres, r5.tr, tt5 );         
        fprintf(fids2,' && %2.1e & %8.0f ', r5.maxres, tt5 );         

        fprintf('\n');
    end    
    
    
    %---------------------------------------------------------------------------------------
    %fprintf('\n\n--- mpm running --- \n')
    if run_mpm
        %opts = [];
        opts.tol = tol;
        opts.idisp = 1;
        opts.mpmsolver = @mpmo;
        opts.innsolver = @mpmirr;
        opts.matname = strcat(name,'-mpm-',eigstr,'-tol',num2str(log10(tol)));
        nAX = 0;
        t0 = tic; 
        [U2,D2,out2] = arrabit(@funA, n, k,'LA',opts);
        tt2 = toc(t0); t2(dprob) = tt2;
        r2 = comp_stat_eig(B, U2, stopRule, 1);
        
        fprintf('MPM  res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r2.maxres, r2.tr, tt2, nAX);
        
        fprintf(fid,'MPM  res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r2.maxres, r2.tr, tt2, nAX);
        
        fprintf(fids,'& %2.1e & %9.8e & %8.0f ', r2.maxres, r2.tr, tt2 );         
        fprintf(fids2,'&& %2.1e & %8.0f ', r2.maxres, tt2 );         

        fprintf('\n');
    end
  
    
    
    if run_GN
        opts.tol = tol;
        opts.idisp = 1;
        opts.mpmsolver = @mpmo;
        opts.innsolver = @GN_slrp;
        opts.matname = strcat(name,'-SLRP-',eigstr,'-tol',num2str(log10(tol)));
        nAX = 0;
        t0 = tic; 
        [U6,D6,out6] = arrabit(@funA, n, k,'LA',opts);
        tt6 = toc(t0); t6(dprob) = tt6;
        r6 = comp_stat_eig(B, U6, stopRule, 1);
        
        fprintf('GN   res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r6.maxres, r6.tr, tt6, nAX);
        
        fprintf(fid,'GN   res: %2.1e, TR = %7.6e, cpu: %8.0f, SpMV: %d\n',...
            r6.maxres, r6.tr, tt6, nAX);
       
        fprintf(fids,' & %2.1e & %9.8e & %8.0f  ', r6.maxres, r6.tr, tt6 );         
        fprintf(fids2,'&& %2.1e & %8.0f ', r6.maxres, tt6 );         

        fprintf('\n');            
        
    end
    
    fprintf(fids, '    \\\\ \\hline \n');        
    fprintf(fids2, ' \\\\  \n'); 
end
fprintf(fids2, ' \\hline \n');
fclose(fid);





